/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * Ladder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package ws.ladder.model;

public final class Edge {
	final Node source;         
	final Node target;   
	final Property property;

	public Edge(Node source, Node target, Property property) {
		this.source = source;
		this.target = target;
		this.property = property;
	}

	public Property getProperty() {
		return property;
	}
	
	public Node getSource() {
		return source;
	}

	public Node getTarget() {
		return target;
	}

	public boolean isRequired() {
		return AbstractProperty.isRequired( property );
	}

	public boolean isCascaded() {
		return AbstractProperty.isCascadable( property );
	}
	
	public boolean fromTransient() {
		boolean result = !getSource().isPersistent();
		
		// Check the owner of the collection
		if(property == null) {
			Node collection = getSource();
			if(collection.getIncoming().size() == 1) {
				Edge edgeFromOwner = collection.getIncoming().iterator().next();
				result = !edgeFromOwner.source.isPersistent();
			}
		}
		
		return result;
	}

	public void link() {
		target.addIncomingEdge(this);
		source.addOutgoingEdge(this);	
	}

	public void unlink() {
		target.removeIncomingEdge(this);
		source.removeOutgoingEdge(this);
	}	
	
	public void disconnect() {
		((ExtendedProperty)property).setValue(source, null);
		unlink();
	}
	
	public void connect() {
		link();
		((ExtendedProperty)property).setValue(source, target.getInstance());
	}	

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 0;
		result = prime * result + ((source == null) ? 0 : System.identityHashCode(source));
		result = prime * result + ((target == null) ? 0 : System.identityHashCode(target));
		result = prime * result + ((property == null) ? 0 : System.identityHashCode(property));			
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if(obj == null)
			return false;
		if (this == obj)
			return true;
		if (getClass() != obj.getClass())
			return false;
		Edge other = (Edge) obj;

		return source == other.source  &&
				target == other.target &&
				property == other.property;
	}

}