/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * Ladder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package ws.ladder.model;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.persistence.Access;
import javax.persistence.AccessType;
import javax.persistence.metamodel.Attribute;
import javax.persistence.metamodel.EmbeddableType;
import javax.persistence.metamodel.EntityType;
import javax.persistence.metamodel.SingularAttribute;
import javax.persistence.metamodel.Type.PersistenceType;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

import ws.ladder.service.JPADataAccessService;

public class JPAType extends AbstractType {
	private static final Logger logger = LogManager.getLogger(new Exception().getStackTrace()[0].getClassName());	

	private javax.persistence.metamodel.Type<?>  persistenceType;
	private List<Type>               baseTypes;
	private AccessType               accessType;
	private JPAProperty              identifierProperty;
	private JPAProperty              versionProperty;	

	public JPAType(javax.persistence.metamodel.Type<?> entityType) {
		super();		
		this.persistenceType = entityType;

		initImmutable();
	}

	@Override
	public String getName() {
		// The name is unique in the namespace because is qualified by the package name
		return persistenceType.getJavaType().getName();
	}

	@Override
	public String getEntityName() {
		if(EntityType.class.isAssignableFrom(persistenceType.getClass()))
			return ((EntityType)persistenceType).getName();
		else
			return getName();
	}	

	protected Iterator<?> getPropertyIterator() {
		if(persistenceType.getPersistenceType() == javax.persistence.metamodel.Type.PersistenceType.ENTITY) {
			return ((EntityType<?>)persistenceType).getAttributes().iterator();
		} else if(persistenceType.getPersistenceType() == javax.persistence.metamodel.Type.PersistenceType.EMBEDDABLE) {
			return ((EmbeddableType<?>)persistenceType).getAttributes().iterator();
		} else {
			throw new UnsupportedOperationException();
		}
	}	

	@Override
	public List<Type> getEmbeddableTypes() {
		List<Type> result = new ArrayList<Type>();

		Iterator<?> itr = getPropertyIterator();
		while(itr.hasNext() ) {
			Attribute<?, ?> attribute = (Attribute<?, ?>) itr.next();
			if(attribute.getPersistentAttributeType() == javax.persistence.metamodel.Attribute.PersistentAttributeType.EMBEDDED) {
				JPAType type = new JPAType(((SingularAttribute<?, ?>)attribute).getType());
				result.add(type);
			}
		}

		return result; 
	}	

	@Override
	public String getURI() {
		return null;
	}

	@Override
	public Class<?> getInstanceClass() {
		return persistenceType.getJavaType();
	}

	@Override
	public boolean isInstance(Object object) {
		return getInstanceClass().isAssignableFrom(object.getClass());
	}

	public void setProperty(JPADataAccessService dataAccessService) {
		if(properties == null) {
			// populate the properties for this type
			properties = new HashMap<String, Property>();	
			Iterator<?> attribIter = getPropertyIterator();
			while(attribIter.hasNext()) {
				Attribute<?, ?> attribute = (Attribute<?, ?>) attribIter.next();
				logger.debug("[" + getName() + "] JPA Property name: " + attribute.getName() + ", type name: " + attribute.getJavaType());

				Type propertyType = dataAccessService.getType(attribute.getJavaType());
				JPAProperty property = new JPAProperty(attribute, propertyType, this);
				if(property.isIdentifier()) {
					identifierProperty = property;
					logger.debug("JPA Identifier attribute name: " + identifierProperty.getName());
				}
				if(property.isVersion()) {
					versionProperty = property;
					logger.debug("JPA version attribute name: " + versionProperty.getName());
				}
				property.init(dataAccessService);
				properties.put(property.getName(), property);
			}		
		} 		

		initAccessType();
	}

	public void setOpposite(JPADataAccessService dataAccessService) {
		for(Property property: properties.values()) {
			((JPAProperty)property).initMappedBy(dataAccessService);
		}
	}

	@Override
	public boolean isOpen() {
		return false;
	}

	@Override
	public boolean isSequenced() {
		return false;
	}

	@Override
	public boolean isAbstract() {
		return false;
	}

	@Override
	public List<Type> getBaseTypes() {
		return baseTypes;
	}

	public void setBaseType(List<Type> types) {
		baseTypes = types;
	}

	@Override
	public List<Property> getDeclaredProperties() {
		List<Property> result = new ArrayList<Property>();

		if(getEntityType().getPersistenceType() != javax.persistence.metamodel.Type.PersistenceType.EMBEDDABLE) {		
			Set<?> declaredAttributes = ((EntityType<?>)persistenceType).getDeclaredAttributes();
			Iterator<?> attribIter = declaredAttributes.iterator();		
			while(attribIter.hasNext()) {
				Attribute<?, ?> attribute = (Attribute<?, ?>) attribIter.next();
				logger.debug("[" + getName() + "] JPA declared property name: " + attribute.getName());
				result.add(properties.get(attribute.getName()));
			}
		}

		return result;
	}

	@Override
	public List<?> getAliasNames() {
		return new ArrayList<String>();
	}

	@Override
	public List<?> getInstanceProperties() {
		return new ArrayList<Object>();
	}

	@Override
	public Object get(Property property) {
		return null;
	}

	public javax.persistence.metamodel.Type<?> getEntityType() {
		return this.persistenceType;
	}

	protected void initAccessType() {
		Access accessAnno = getInstanceClass().getAnnotation( Access.class );
		if ( accessAnno != null ) {
			accessType = accessAnno.value();
		}

		if(accessType == null)
			accessType = inferAccessType();
	}

	@Override
	public AccessType getAccessType() {
		return accessType;
	}	

	protected AccessType inferAccessType() {
		boolean hasAccessAnnotation = false;

		// first check if any of its properties have the Access annotation
		for(Property property: properties.values()) {
			if( ((AbstractProperty)property).getAccessType() != null) {
				hasAccessAnnotation = true;
				break;
			}
		}

		if(!hasAccessAnnotation) {
			for(Property property: properties.values()) {
				if(((JPAProperty)property).isFieldMapped())
					return AccessType.FIELD;
				else if(((JPAProperty)property).isPropertyMapped())
					return AccessType.PROPERTY;
			}
		}

		// Should not come here
		throw new IllegalStateException("The persistent class does not have any JPA annotations");
	}

	@Override
	public Property getIdentifierProperty() {
		return identifierProperty;
	}

	@Override
	public boolean isEmbedded() {
		return persistenceType.getPersistenceType() == PersistenceType.EMBEDDABLE; 
	}
	
	@Override
	public boolean isEntity() {
		return persistenceType.getPersistenceType() == PersistenceType.ENTITY; 
	}	

	@Override
	public Property getVersionProperty() {
		return versionProperty;
	}
}
