/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * Ladder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package ws.ladder.service;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.BeanCreationException;

import ws.ladder.model.MapperDirection;
import ws.ladder.model.TypeMapper;
import ws.ladder.util.AccessType;

public abstract class AbstractDASFactory implements DASFactory {
	private static final Logger logger = LogManager.getLogger(new Exception().getStackTrace()[0].getClassName());

	protected String                                name;	
	protected AggregateManager                      aggregateManager;
	protected static Map<String, DataAccessService> das = new ConcurrentHashMap<String, DataAccessService>(); // If the name is specified then the DAS is cached by the name

	public AggregateManager getAggregateManager() {
		return aggregateManager;
	}

	public void setAggregateManager(AggregateManager aggregateManager) {
		this.aggregateManager = aggregateManager;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	/**
	 * Need to synchronize on this method, since the meta model needs to be ready before using Ladder
	 */
	public synchronized DataAccessService create() {
		if(name == null)
			throw new RuntimeException("Name needs to be specified for the DASFactory");

		if(das.get(name) != null)
			return das.get(name);

		synchronized(this.getClass()) {

			// Needed for creating the derived types
			TypeMapper typeMapper = aggregateManager.getTypeMapper().newInstance(MapperDirection.REFERENCETODERIVED);
			AccessType accessType = aggregateManager.getAccessType();

			try { // HibernateDAS
				if(accessType == null || accessType == AccessType.HIBERNATE) {
					das.put(name, new HibernateDAS(typeMapper, this));
					injectDependencies(das.get(name), name);

					// Try version 4 specifically
					if( ((HibernateDAS)das.get(name)).getConfiguration() == null) {
						das.put(name, new Hibernate4DAS(typeMapper, this));
						injectDependencies(das.get(name), name);
					}
					
					if( ((HibernateDAS)das.get(name)).getConfiguration() == null)
						throw new RuntimeException("Could not get Hibernate configuration.");

					das.get(name).define();
					return das.get(name);
				}
			} catch(BeanCreationException e) {
				logger.warn("Hibernate configuration not found, hence cannot create a HibernateDAS instance");			
			}

			// This solution is not yet ready - should be ready in Hibernate 5
			try { // Hibernate configured using MetadataSources and ServiceRegistry
				if(accessType == null || accessType == AccessType.METADATA) {
					das.put(name, new MetadataDAS(typeMapper, this));
					injectDependencies(das.get(name), name);
					das.get(name).define();
					return das.get(name);
				}
			} catch (BeanCreationException e) {
				logger.warn("Hibernate 5.0+ configuration not found, hence cannot create a MetadataDAS instance");
			}	

			try { // JPADataAccessService
				if(accessType == null || accessType == AccessType.JPA) {
					das.put(name, new JPADataAccessService(typeMapper, this));
					injectDependencies(das.get(name), name);
					das.get(name).define();
					return das.get(name);
				}
			} catch (BeanCreationException e) {
				logger.warn("JPA configuration not found, hence cannot create a JPADataAccessService instance");
			}

			try { // MongoDB, use JPA for metadata configuration
				if(accessType == null || accessType == AccessType.MONGODB) {
					das.put(name, new JPADataAccessService(typeMapper, this));
					injectDependencies(das.get(name), name);
					das.get(name).define();
					return das.get(name);
				}
			} catch (BeanCreationException e) {
				logger.warn("MongDB configuration not found");
			}	

			try { // Riak, use JPA for metadata configuration
				if(accessType == null || accessType == AccessType.RIAK) {
					das.put(name, new JPADataAccessService(typeMapper, this));
					injectDependencies(das.get(name), name);
					das.get(name).define();
					return das.get(name);
				}
			} catch (BeanCreationException e) {
				logger.warn("Riak configuration not found");
			}	
		}

		return das.get(name);
	}

	@Override
	public PersistenceOrchestrator getPersistenceOrchestrator() {
		PersistenceOrchestrator result;

		if(das.get(name) == null)
			this.create();

		switch(das.get(name).getAccessType()) {
		case HIBERNATE:
		case METADATA:			
			result = new HibernatePersistenceOrchestrator();
			break;
		case JPA:
			result = new JPAPersistenceOrchestrator();
			break;
		case MONGODB:
			result = new MongoDBPersistenceOrchestrator();
			break;
		case RIAK:
			result = new RiakPersistenceOrchestrator();
			break;
		default:
			result = null;
			break;
		}

		if(result != null)
			injectDependencies(result, null);

		return result;
	}
}
