/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * Ladder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package ws.ladder.service;

import java.io.Serializable;
import java.util.List;
import java.util.Map;

import javax.inject.Inject;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.PersistenceContext;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import ws.ladder.model.Type;
import ws.ladder.model.view.JPAQuery;
import ws.ladder.model.view.Query;

public class JPAPersistenceOrchestrator extends AbstractPersistenceOrchestrator {
	@PersistenceContext
	private EntityManager entityManager;
	
    @Inject
    EntityManagerFactory entityManagerFactory;
	
	/**
	 * Save the entity in the persistence store
	 * 
	 * @param entity
	 * @return
	 */
	@Override
	public void saveOrUpdate(Object entity) {
		entityManager.persist(entity);		
	}

	/**
	 * Delete the entity from the persistence store
	 * 
	 * @param entity
	 */
	@Override
	public void delete(Object entity) {
		entityManager.remove(entity);		
	}

	@Override
	public void flush() {
		entityManager.flush();
	}

	@Override
	public Object findById(Class<?> persistentClass, Object id) {
	   return entityManager.find(persistentClass, (Serializable) id);
	}
	
	@Override
	public Object findByProperty(Type type, Map<String, String> propertyValues) {
		CriteriaBuilder builder = entityManagerFactory.getCriteriaBuilder();
		CriteriaQuery<Object> crit = builder.createQuery();
		
		Class<?> persistentClass = type.getInstanceClass();
		Root from = crit.from(persistentClass);
		CriteriaQuery<Object> select = crit.select(from);

		Predicate[] predicates = new Predicate[propertyValues.size()];
		int index = 0;
		for(String property: propertyValues.keySet())
			predicates[index++] = builder.equal(from.get(property), propertyValues.get(property));

		crit.where(builder.and(predicates));

		TypedQuery<Object> typedQuery = entityManager.createQuery(select);
		List<Object> resultList = typedQuery.getResultList();
		if (resultList != null && !resultList.isEmpty()) {
			return resultList.get(0);
		}
		return null;		
	}	

	@Override
	public QueryCapability getQueryCapability() {
		return new JPAQueryCapability();
	}


	@Override
	public Query getQuery(String queryString, boolean isSQL) {
		
		Query result = null;
		if(isSQL) {
			result = new JPAQuery(entityManager.createNativeQuery(queryString));
		} else
			result = new JPAQuery(entityManager.createQuery(queryString));

		return result;
	}		
}