/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * Ladder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package ws.ladder.service;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import javax.annotation.Resource;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.Configuration;
import org.hibernate.cfg.Mappings;
import org.hibernate.metamodel.Metadata;
import org.hibernate.metamodel.MetadataSources;
import org.hibernate.metamodel.binding.EntityBinding;
import org.hibernate.metamodel.domain.Entity;
import org.hibernate.service.ServiceRegistry;
import org.hibernate.service.ServiceRegistryBuilder;
import org.springframework.beans.factory.annotation.Autowired;

import ws.ladder.model.MetadataType;
import ws.ladder.model.SimpleType;
import ws.ladder.model.Type;
import ws.ladder.model.TypeMapper;
import ws.ladder.util.AccessType;

/**
 * This class is part of the Data Access Service framework
 * This implementation will use the MetadataSources information (available from Hibernate 5.0).
 *  
 * @author Dilip Dalton
 *
 */
public class MetadataDAS extends AbstractDataAccessService {

	private static final Logger logger = LogManager.getLogger(new Exception().getStackTrace()[0].getClassName());	

	@Resource(name="&sessionFactory")
	protected Object sessionFactoryBean;
	
	@Autowired
	protected SessionFactory sessionFactory;		
    
	protected Configuration configuration;
	protected Mappings      mappings;	

	public MetadataDAS(TypeMapper typeMapper, DASFactory dasFactory) {
		super(dasFactory);
		this.typeMapper = typeMapper;
	}
	
	public Mappings getMappings() {
		return mappings;
	}
	
	public Configuration getConfiguration()  {
		if(configuration == null) {
			try {
				Method getConfigMethod = sessionFactoryBean.getClass().getMethod("getConfiguration", new Class[0]);
				Configuration config = (Configuration) getConfigMethod.invoke(sessionFactoryBean, new Object[0]);
				setConfiguration(config);
			} catch (Exception e) {
				logger.error("Unable to find the getConfiguration method in the session factory object");
			}
		}
		
		return configuration;
	}	
	
	public void setConfiguration(Configuration configuration) {
		this.configuration = configuration;
	}		

	@Override
	public Type getType(Class<?> clazz) {
		Type result = super.getType(clazz.getName());		

		// create a Type object for this class
		if(result == null) {
			result = new SimpleType(clazz);
			addType(clazz.getName(), result);
		}

		return result;
	}	

	public void define() {

		ServiceRegistry serviceRegistry = new ServiceRegistryBuilder().applySettings(getConfiguration().getProperties()).buildServiceRegistry();
		MetadataSources metadataSources = new MetadataSources(serviceRegistry);
		Metadata metaData = metadataSources.buildMetadata();
		
		logger.debug("Getting the list of hibernate mapped classes");
		Iterable<EntityBinding> bindingsIterator = metaData.getEntityBindings();
		for(EntityBinding entityBinding: bindingsIterator) {
			Entity entity = entityBinding.getEntity();
			logger.debug("     Adding hibernate persisted class: " + entity.getClassName());
			defineTypes(entityBinding);			
		}
		 		
		// Set the base types
		setBaseTypes();		

		// Define the properties for the Types 
		// This will end up defining the simple types
		defineProperties();	
		
		postProcess();		
	}

	protected void defineTypes(EntityBinding entityBinding) {
		MetadataType dataType = new MetadataType(sessionFactory.getTypeHelper().entity(entityBinding.getEntity().getClassName()),
				entityBinding);
		logger.debug("Defined data type: " + dataType.getName());
		addType(dataType.getName(), dataType);
	}

	protected void defineProperties() {
		for(Type type: types.values()) {
			if(MetadataType.class.isAssignableFrom(type.getClass())) {
				MetadataType hibernateType = (MetadataType) type;
				hibernateType.setProperty(this);
			}
		}

		// Link the bi-directional relationship between the properties
		for(Type type: types.values()) {
			if(MetadataType.class.isAssignableFrom(type.getClass())) {
				MetadataType hibernateType = (MetadataType) type;
				hibernateType.setOpposite(this);
			}			
		}		
	}	

	protected void setBaseTypes() {
		for(Type type: types.values()) {
			if(MetadataType.class.isAssignableFrom(type.getClass())) {
				MetadataType hibernateType = (MetadataType) type;

				List<Type> baseTypes = new ArrayList<Type>();
				EntityBinding superEntityBinding = hibernateType.getEntityBinding().getSuperEntityBinding();
				if(superEntityBinding != null) {
					Type baseType = types.get(superEntityBinding.getEntity().getClassName());
					if(baseType != null)
						baseTypes.add(baseType);
				}
				hibernateType.setBaseType(baseTypes);
			}
		}
	}

	@Override
	public List<String> getAggregateList() {
		// TODO Fix me
		return new ArrayList<String>();
	}

	@Override
	public AccessType getAccessType() {
		return AccessType.METADATA;
	}
}